/*********************************************************************
 *
 *                  Microchip PICkit 2 v2 Flash Starter Kit
 *
 *********************************************************************
 * FileName:        pk_isr.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 3.00
 * Company:         Microchip Technology, Inc. / JAKA
 * Homepage:        http://kair.us/projects/pk2m_programmer/
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Walter Kicinski   2006-May-15    Initial Write
 * Jaakko Kairus     19.2.2022      Adapted to PIC18F25K50 for PK2M
 ********************************************************************/

/** I N C L U D E S **********************************************************/

#include <p18cxxx.h>
#include "io_cfg.h"
#include "system\typedefs.h"
#include "user\pk_isr.h"
#include "pickit.h"


/** V A R I A B L E S ********************************************************/



/** P R I V A T E  P R O T O T Y P E S ***************************************/


/** D E C L A R A T I O N S **************************************************/

#pragma code

/******************************************************************************
 * Function:        
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        
 *
 * Note:            Refer to PIC18F2455/2550/4455/4550 Errata concerning
 *                  high-priority interrupt problem.
 *   
 *****************************************************************************/

#pragma interruptlow InterruptHandler

void InterruptHandler (void)
{
	if (Pk2Status.UARTMode == 1)
    {
     
   		if (PIR1bits.RCIF) {
			PIR1bits.RCIF = 0;
            
            UARTStatus.RXByteArray[UARTStatus.NewRX] = RCREG1;
            
	        if (UARTStatus.NewRX < 11) {
            	UARTStatus.NewRX++;
            }	
		}
	}
	else {
	    
        if (PIR1bits.TMR1IF)        // Timer 1 ADC voltage monitor
        {
            PIR1bits.TMR1IF = 0;    // clear flag
            TMR1H = Timer1_Timeout_High;  // reset timer
            TMR1L = Timer1_Timeout_Low;
    
            // CHECK VDD VOLTAGE
            //if (ADCON0 == ADC_VDD_Tgt_Ch) // last conversion was Vdd (should always be)
            //{
    			//if ((!Vdd_TGT_P_pin) && (ADRESH < VddVppLevels.VddThreshold))  // Don't check for short if not on.
    			if ((!Vdd_TGT_P) && (ADRESH < VddVppLevels.VddThreshold))  // Don't check for short if not on.
    				{
    				// Conversion for VDD less than threshold.  Assume Short/heavy load.
    				// Must get MaxVError such conversions to minimize shutting off power due to glitches, ramp-up
    				if (VddVppLevels.VddErrCount == MaxErrorVDD)
    					{
    						Vpp_ON = 0;		// shut off VPP 
    	                    Vdd_TGT_P = 1;	// shut off VDD as well
    	                    tris_Vdd_TGT_P = 1;				// Tri-state to prevent leakage with Vdd_TGT < 5V.
                        	//#!# BusyLED_BlinkOn();
    	                    Pk2Status.VddError = 1;		// indicate error in status
    					}
    				else
    					{
    					VddVppLevels.VddErrCount++;			// Increment on error.
    					} // end if (VddVppLevels.VddErrCount == n)
    				}
    			else
    				{
    				VddVppLevels.VddErrCount = 0;				// Reset on good voltage
    				} // end if ((!Vdd_TGT_P_pin) && (ADRESH < VddVppLevels.VddThreshold))
            //} 
            if (Pk2Status.StatusLow & 0x30) // if Pk2Status.VddError or Pk2Status.VppError is set
            {
                // blink Busy at 12 Herz on error.
                if (VddVppLevels.BlinkClount++ > BusyBlinkRate)
                {
                    BUSY_LED = !BUSY_LED;
                    VddVppLevels.BlinkClount = 0;
                }
            }
    
            PIR1bits.ADIF = 0; // clear flag
            ADCON0 = ADC_VPP_FB_Ch;			    // Check VPP next.
            PIE1bits.ADIE = 1;                  // interrupt when done
            ADCON0bits.GO = 1;          		// start conversion
        } // end if (PIR1bits.TMR1IF)
    
    
        if (PIR1bits.ADIF)  // VPP conversion interrupt
        {
            PIR1bits.ADIF = 0; // clear flag
    
           	// CHECK VPP VOLTAGE
            //if (ADCON0 == ADC_VPP_FB_Ch) // last conversion was VPP (presently only ADIF interrupt)
            //{
    			// Maintain VPP voltage by adjusting PWM on/off
    			if (CCP2CON == VppPWM_Enable)	// skip if PWM not on
    				{
    				if (ADRESH > Vpp_PWM.UppperLimit)
    					{
    					CCPR2L = VppDutyCycleOff;
    					}
    				if (ADRESH < Vpp_PWM.LowerLimit)
    					{
    					CCPR2L = Vpp_PWM.CCPRSetPoint;
    					// CCPR2L = 40;	// static value for testing
    					}	
        			// Check for short or heavy load
        			if ((Vpp_ON_pin == 1) && (ADRESH < VddVppLevels.VppThreshold))  // Don't check for short if not on
        				{
        				// Conversion for VPP less than threshold.  Assume Short/heavy load.
        				// NOTE: Q6 will go into active mode when VPP is shorted, so VPP_FEEDBACK will drop
        				// by less percentage-wise the lower VPP gets.
        				// Must get MaxVError such conversions to minimize shutting off power due to glitches, ramp-up
        				if (VddVppLevels.VppErrCount == MaxErrorVPP)
        					{
        						Vpp_ON = 0;		// shut off VPP 
        	                    Vdd_TGT_P = 1;	// shut off VDD as well
        	                    tris_Vdd_TGT_P = 1;				// Tri-state to prevent leakage with Vdd_TGT < 5V.
                        		// #!# BusyLED_BlinkOn();
        	                    Pk2Status.VppError = 1;		// indicate error in status
        					}
        				else
        					{
        					VddVppLevels.VppErrCount++;			// Increment on error.
        					} // end if (VddVppErrCounts.VPP)
        				}
        			else
        				{
        				VddVppLevels.VppErrCount = 0;				// Reset on good voltage
        				} // end if ((Vpp_ON_pin == 1) && (ADRESH < VddVppLevels.VppThreshold))
                    } //end if  (CCP2CON = VppPWM_On)
            //} // end if (ADCON0 == ADC_VDD_Tgt_Ch) else if (ADCON0 == ADC_VPP_FB_Ch)
            ADCON0 = ADC_VDD_Tgt_Ch;		// Check VDD next.
            PIE1bits.ADIE = 0;              // don't interrupt - wait for Timer1 int
            ADCON0bits.GO = 1;          	// start conversion
        } // end if (PIR1bits.ADIF)
    }
    /* // Commented out 8.1.2023. Also setup commented from begin of pickit.c!
    if (PIR2bits.HLVDIF)
    {
        Reset();
    }
	*/
} // end void InterruptHandler(void)

#pragma code

/******************************************************************************
 * Function: void VppVddADCTmr1_Start(void)       
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          Starts Timer1 driving ADC conversions of VPP/VDD
 *
 * Side Effects:    Uses Timer1 and ADC
 *
 * Overview:        
 *
 * Note:            On a Timer1 interrupt, the results of the last VDD conversion are read
 *                  and a VPP conversion is started also enabling the ADC interrupt.
 *                  When the VPP conversion is finished, it interrupts, the results are
 *                  handled, and a VDD conversion is started also disabling the ADC interrupt.
 *                  The VDD result will not be read until the next Timer1 interrupt.
 *   
 *****************************************************************************/
void VppVddADCTmr1_Start(void)
{

	// Init Error Counts
	VddVppLevels.VddErrCount = 0;
	VddVppLevels.VppErrCount = 0;

    // Set up Timer 1
    T1CON = ADCMonitorTmr1_Setup;
    TMR1H = Timer1_Timeout_High;
    TMR1L = Timer1_Timeout_Low;
    PIR1bits.TMR1IF = 0;            // clear int flag
    PIE1bits.TMR1IE = 1;            // enable int
    T1CONbits.TMR1ON = 1;           // start timer

	// Assumes RA0/AN0 and RA1/AN1 set to analog by PICkitInit() and chip VDD/VSS selected as references.
    PIR1bits.ADIF = 0;              // clear A/D Converter Interrupt Flag
    PIE1bits.ADIE = 0;              // ensure A/D Converter Interrupt disabled
	ADCON2 = ADC_LftJust_8TAD_FoscDiv64;	// 8 TAD at Fosc/64.
    ADCON0 = ADC_VDD_Tgt_Ch;
    ADCON0bits.GO = 1;                      // start conversion.

}

/******************************************************************************
 * Function: void VppVddADCTMR1_Stop(void)       
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          Shuts off Timer1, disables interrupt, and stops ADC.
 *
 * Side Effects:    None
 *
 * Overview:        
 *
 * Note:            
 *   
 *****************************************************************************/
void VppVddADCTMR1_Stop(void)
{
    // disable timer1
    PIE1bits.TMR1IE = 0;            // disable int
    T1CONbits.TMR1ON = 0;           // stop timer

	// Shut off ADC conversion in progress.
	ADCON0 = 0;

    // disable ADC
    PIR1bits.ADIF = 0;              // clear A/D Converter Interrupt Flag
    PIE1bits.ADIE = 0;              // ensure A/D Converter Interrupt disabled
}


/** EOF pk_isr.c ************************************************************/
